unit MainFormU;

interface

uses
  Androidapi.JNI.GraphicsContentViewText,
  Androidapi.JNI.Nfc,
  System.SysUtils, System.Types, System.UITypes, System.Classes, System.Variants,
  FMX.Types, FMX.Controls, FMX.Forms, FMX.Graphics, FMX.Dialogs, FMX.StdCtrls, FMX.Layouts, FMX.ListBox, NFCHelper,
  FMX.Edit;

type
  TMainForm = class(TForm)
    NFCTagIdLabel: TLabel;
    InfoLabel: TLabel;
    PromptLabel: TLabel;
    TagWriteEdit: TEdit;
    TagWriteButton: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure TagWriteButtonClick(Sender: TObject);
  private
    { Private declarations }
    NfcAdapter: JNfcAdapter;
    procedure RegisterDelphiNativeMethods;
  public
    { Public declarations }
    procedure OnNewIntent(Intent: JIntent);
  end;

var
  MainForm: TMainForm;

implementation

{$R *.fmx}

uses
  FMX.Helpers.Android,
  Androidapi.NativeActivity,
  Androidapi.JNIBridge,
  Androidapi.JNI,
  Androidapi.JNI.JavaTypes,
  Androidapi.JNI.Os,
  Androidapi.JNI.Toast;

{ TMainForm }

{$REGION 'JNI setup code and callback'}
procedure OnNewIntentNative(PEnv: PJNIEnv; This: JNIObject; NewIntent: JNIObject); cdecl;
var
  LocalIntent: JIntent;
begin
  Log.d('Queuing native routine to run synchronized');
  LocalIntent := TJIntent.Wrap(NewIntent);
  TThread.Queue(nil,
    procedure
    begin
      Log.d('+ThreadSwitcher');
      Log.d('Thread: Main: %.8x, Current: %.8x, Java:%.8d (%2:.8x)', [MainThreadID, TThread.CurrentThread.ThreadID,
        TJThread.JavaClass.CurrentThread.getId]);
      MainForm.OnNewIntent(LocalIntent);
      Log.d('-ThreadSwitcher');
    end);
end;

procedure TMainForm.RegisterDelphiNativeMethods;
var
  PEnv: PJNIEnv;
  ActivityClass: JNIClass;
  NativeMethod: JNINativeMethod;
begin
  Log.d('Starting the registration JNI stuff');

  PEnv := TJNIResolver.GetJNIEnv;

  Log.d('Registering interop method');

  NativeMethod.Name := 'onNewIntentNative';
  NativeMethod.Signature := '(Landroid/content/Intent;)V';
  NativeMethod.FnPtr := @OnNewIntentNative;

  ActivityClass := PEnv^.GetObjectClass(PEnv,
    PANativeActivity(System.DelphiActivity).clazz);

  PEnv^.RegisterNatives(PEnv, ActivityClass, @NativeMethod, 1);

  PEnv^.DeleteLocalRef(PEnv, ActivityClass);
end;
{$ENDREGION}

procedure TMainForm.FormCreate(Sender: TObject);
begin
  Log.d('OnCreate');
  RegisterDelphiNativeMethods;
  NfcAdapter := TJNfcAdapter.JavaClass.getDefaultAdapter(SharedActivityContext);
  if NfcAdapter = nil then
  begin
    // Could do with exiting here maybe...
    raise Exception.Create('No NFC adapter present');
  end;
end;

procedure TMainForm.FormActivate(Sender: TObject);
var
  Intent: JIntent;
begin
  Log.d('OnActivate');
  if NfcAdapter <> nil then
  begin
    if not NfcAdapter.isEnabled then
    begin
      Toast('NFC is not enabled.' + LineFeed + 'Launching NFC settings.');
      SharedActivity.startActivity(TJIntent.JavaClass.init(StringToJString('android.settings.NFC_SETTINGS')));
    end;
  end;
  Intent := SharedActivity.getIntent;
  if not TJIntent.JavaClass.ACTION_MAIN.equals(Intent.getAction) then
  begin
    Log.d('Passing along received intent');
    OnNewIntent(Intent);
  end;
end;

procedure TMainForm.OnNewIntent(Intent: JIntent);
var
  TagParcel: JParcelable;
  Tag: JTag;
begin
  SharedActivity.setIntent(Intent);
  if Intent <> nil then
  begin
    Log.d('Intent action = %s', [JStringToString(Intent.getAction)]);
    if TJNfcAdapter.JavaClass.ACTION_NDEF_DISCOVERED.equals(Intent.getAction) or
       TJNfcAdapter.JavaClass.ACTION_TECH_DISCOVERED.equals(Intent.getAction) or
       TJNfcAdapter.JavaClass.ACTION_TAG_DISCOVERED.equals(Intent.getAction) then
    begin
      PromptLabel.Visible := False;
      Log.d('Getting Tag parcel from the received Intent');
      TagParcel := Intent.getParcelableExtra(TJNfcAdapter.JavaClass.EXTRA_TAG);
      if TagParcel <> nil then
      begin
        Log.d('Wrapping tag from the parcel');
        Tag := TJTag.Wrap((TagParcel as ILocalObject).GetObjectID);
      end;
      InfoLabel.Text := '';
      NFCTagIdLabel.Text := HandleNfcTag(Tag,
        procedure (const Msg: string)
        begin
          Log.d('Adding to UI: ' + Msg);
          InfoLabel.Text := InfoLabel.Text + Msg + LineFeed;
        end);
    end;
  end;
end;

procedure TMainForm.TagWriteButtonClick(Sender: TObject);
var
  TagParcel: JParcelable;
  Tag: JTag;
  Intent: JIntent;
begin
  if (NfcAdapter <> nil) and NfcAdapter.isEnabled then
  begin
    Intent := SharedActivity.getIntent;
    TagParcel := Intent.getParcelableExtra(TJNfcAdapter.JavaClass.EXTRA_TAG);
    if TagParcel <> nil then
    begin
      Log.d('Wrapping tag from the parcel');
      Tag := TJTag.Wrap((TagParcel as ILocalObject).GetObjectID);
      if not WriteTagText(TagWriteEdit.Text, Tag) then
        raise Exception.Create('Error connecting to tag');
    end;
  end
  else
    raise Exception.Create('NFC is not available');
end;

end.
